/*
 * libusb example program to manipulate Fcad416 analog-to-digital converter.
 * Copyright (C) 2009 Kentaro Hayashi <linux-support@flexcore.jp>
 *
 * This sample is basic operation only,so it open unit and set parameters
 * then start sampling and get data.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include <errno.h>
#include <signal.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>		// for sleep function

#include <libusb/libusb.h>
#include <DLLFunc.H>
#include <Fcad416DEF.H>
#include <define.h>

#define EP_INTR			(1 | LIBUSB_ENDPOINT_IN)				// 81H
#define EP_DATA			(2 | LIBUSB_ENDPOINT_IN)				// 82H
#define CTRL_IN			(LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_ENDPOINT_IN)	// C0H
#define CTRL_OUT		(LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_ENDPOINT_OUT)	// 40H
#define USB_RQ			0x04
#define INTR_LENGTH		64
#define	Vend_Id			0x20F3				// Flexcore's vendor ID
#define	Prod_Id			0x0000				// Fcad416's product ID
#define	MAX_BOARD	16
#define	PriTrigBufSize	65536
#define	Max_ScanSpeed	4
#define	BlockSize		256

static struct libusb_device_handle *devh = NULL;
static unsigned char AD_ReadBuf[0x20000];						// 64KWord memory
int	samplingclockvalue;											// TvONbNݒl
int	samplingnumber;												// TvOf[^
int	scanclocknumber;											// XLNbNilj
int trg_mode, trg_source, trg_type, stop_mode, trig_level1, trig_level2;	// Fcad416_Set_Trigger֐̃p[^
int ringmode, board_no[MAX_BOARD];
int	clk_source, set_mode, time_unit, clk_period, exclk_freq;	// Fcad416_Set_Clock֐̃p[^
int post_samp;													// Fcad416_Start_Samp֐̃p[^
int post_sampled, pre_sampled, pre_position;					// 擾ς݃f[^
int no_ch, scan_order[MAX_BOARD], range[MAX_BOARD];				// Fcad416_Set_SampCh֐̃p[^
int inp_mode, ad_code;											// Fcad416_Set_InpMode֐̃p[^
int multi;														// Fcad416_Set_MultiScan֐̃p[^
int	transfersize;												// Fcad416_Set_TransferSize֐̃p[^
int ScanClockMode, Scanclockvalue;								// Fcad416_Set_ScanSpeed֐̃p[^
int	Memory_size[MAX_BOARD];										// Fcad416_Set_Memory֐̃p[^
int MessageRequest[MAX_BOARD];									// bZ[WvێWX^
int	OpenedBoard[MAX_BOARD];										// I[v{[hL^ϐ
int	CheckParameterSet[MAX_BOARD];								// K{p[^ZbgꂽǂL^ĂWX^
int	SelectedBoard;												// Iꂽ{[h̃[^[XCb`ԍ
int	TestCount = 0;												// 񐔃JE^[
int	InSamplingMode;												// TvOǂ𔻒肷ϐ
int	data_count;
int	getdata_number, fifodata_number;
unsigned char	Startmode[MAX_BOARD];							// TvO[hێϐ
unsigned short	RotarySwVal[MAX_BOARD];							// [^[XCb`̒lێz
unsigned char	TrigValue[MAX_BOARD];							// gK[hۑĂWX^
unsigned char	TrigSet[MAX_BOARD];								// gK[hݒL^ĂWX^
unsigned char	Scan_Set[MAX_BOARD];							// `lݒIL^郌WX^
int	channels[MAX_BOARD];										// TvO`lێϐ
unsigned char	ClockValue[MAX_BOARD];							// NbNݒliEO^EオGbWj
int	PacerClock[MAX_BOARD];
unsigned short	ScanClockValue[MAX_BOARD];
unsigned char	InpMode[MAX_BOARD];								// `cR[hۑWX^
int	transfer_size[MAX_BOARD];									// ]TCYێϐ
unsigned char	HalfFullSize[MAX_BOARD];						// n[ttTCYwf[^
unsigned char	RingBufferSize[MAX_BOARD];						// Oobt@TCYێWX^
int	pre_sampled_chnum[MAX_BOARD], transfered_data_num[MAX_BOARD];
unsigned char	Image_Status[MAX_BOARD][2];
unsigned char	Image_Aux_in[MAX_BOARD];
unsigned char	ServiceRequestStatus[MAX_BOARD];
int	last_memory_size[MAX_BOARD], pri_last_memsize[MAX_BOARD];
int	Wait_to_Transmit[MAX_BOARD], memory_size[MAX_BOARD];
int	pri_memsize[MAX_BOARD];
int Fcad416_Get_Status(int *post_sampled_data, int *pre_sampled_data, int get_status[]);
static int Fcad416_StatusCheck( void );

static int find_flexcore_device(void)
{
	devh = libusb_open_device_with_vid_pid(NULL, Vend_Id, Prod_Id);
	return devh ? 0 : -EIO;
}

static int Hard_Init(void)
{
	int r, i, elasp_time;
	unsigned char	ad_status[2];

	for (i = 0; i < MAX_BOARD; i++){			// foCXpϐiőPUΉj
		transfer_size[i] = 128;				// ftHg͍ő]f[^iPQWoCgUSjoCgɕύXj
		transfered_data_num[i] = 0;
		channels[i] = 1;
		last_memory_size[i] = 0;
		pri_last_memsize[i] = 0;
		Startmode[i] = LIMIT;					// ftHg͗LTvO
		TrigValue[i] = 0;
		Wait_to_Transmit[i] = 0;
		TrigSet[i] = Not_Trigged;				// gK[hݒԂɏ
		CheckParameterSet[i] = 0;
		InpMode[i] = 0;
		HalfFullSize[i] = 0x60;					// ftHg̃n[ttTCYPQWoCgɐݒ
		RingBufferSize[i] = 0x0;				// ftHg̃Oobt@TCY͂O
//		Message_Control[i] = 0;					// ftHg͍ŒჁbZ[ŴݑM
//		EmergencyStop[i] = OFF;
//		WithOutTimeOut[i] = OFF;				// ^CAEgԁi800mSȏ)ɏ
		RotarySwVal[i] = 0xff;					// [^[XCb`̐ݒlۑGȀ
		board_no[i] = RotarySwVal[i];			// ꂽ[^[XCb`ԍU[U[obt@ɃRs[
//		EnumRotarySwVal[i] = 0;					// [^[XCb`̒l̏o񐔃JE^[iG[opj
	}
	do{
	r = libusb_control_transfer(devh, CTRL_OUT, Open_Unit, 0, 0, ad_status, 0, 2000);
	}while(r != 0);

	r = libusb_control_transfer(devh, CTRL_IN, Reset_Unit, 0, 0, ad_status, 1, 0);
	if (r < 0){
		fprintf(stderr, "Can't read unit id %d\n", r);
		return r;
	}
	else if(ad_status[0] == 0x2f){
		printf("HardWare initialize ID = %02x\n",ad_status[0]);
		return 0;
	}
	else{
		elasp_time = 0;
		do{
			sleep( 1 );
			elasp_time++;
			r = libusb_control_transfer(devh, CTRL_IN, Reset_Unit, 0, 0, ad_status, 1, 0);
		}while((ad_status[0] != 0x2f) && (elasp_time < 30));
		if (ad_status[0] == 0x2f){
			sleep( 1 );		// In this case need timing for internal hardware initialize
			printf("HardWare initialize ID = %02x\n",ad_status[0]);
			return 0;
		}
		else{
			fprintf(stderr,"Fcad416 absent\n");
			return -1;
		}
	}
}

static int Fcad416_Start_Samp( int local_post_samp )
{
	int r, loop, localmatchnum;
	unsigned char	ctrl_buf[64];

	localmatchnum = 0;

	if (post_samp == 0)
		return ERR_SAMPLING_NUMBER;

	if ((Startmode[localmatchnum] & PRE_START) && (RingBufferSize[localmatchnum] == 0))
		return ERR_RingBufferSize;			// vgK[hŁAOobt@eʂOG[񍐂

	if ( CheckParameterSet[localmatchnum] != 0xff )		// Kvȃp[^Sɂ͑ĂȂꍇG[񍐂
		return ~CheckParameterSet[localmatchnum] & 0xff;	// G[ԂԂ

	memory_size[localmatchnum] = post_samp * channels[localmatchnum];			// TvOf[^vZ

	if (Startmode[localmatchnum] & PRE_START){
		switch(RingBufferSize[localmatchnum] & 0x0f){			// vgKobt@݂̂sbNAbv
		case 1:
			pri_memsize[localmatchnum] = 0x100000;
			break;
		case 2:
			pri_memsize[localmatchnum] = 0x200000;
			break;
		case 3:
			pri_memsize[localmatchnum] = 0x300000;
			break;
		case 4:
			pri_memsize[localmatchnum] = 0x400000;
			break;
		case 5:
			pri_memsize[localmatchnum] = 0x500000;
			break;
		case 6:
			pri_memsize[localmatchnum] = 0x600000;
			break;
		case 7:
			pri_memsize[localmatchnum] = 0x700000;
			break;
		default:										// vgKobt@̏ꍇ
			pri_memsize[localmatchnum] = 0x000000;
			break;
		}
		if (pri_memsize[localmatchnum] == 0)
			return ERR_RingBufferSize;						// G[񍐂ďI
	}
	// |XggKobt@̐錾s
	// TCY̓ڈȍ~Ȃ牽Ȃ
	/************************************************************************************************/
	/* vgK[hǂmFAKvȂobt@mۂAsvȃobt@J	*/
	/************************************************************************************************/
	Startmode[localmatchnum] |= START;

	// ONbNgp[h́AɃ^CAEgoIɂBiNbN
	// ΓIɂfłATvOԂ̌ς肪łȂ߁j
	// ܂Ay[T[NbNƃTvO̐ςA^CAEgKlȉ^CAEgoItɂ
		// startmode(Sdat[2]) samplenum(Sdat[4::7])

	for (loop = 0 ; loop < 4 ; loop++)
		ctrl_buf[loop] = (unsigned char)(local_post_samp >> (8 * loop));
	r = libusb_control_transfer(devh, CTRL_OUT, Start_Samp, Startmode[localmatchnum], 0, ctrl_buf, 4, 10);

	if (r < 0){
		fprintf(stderr,"Start Samp is fail.\n");
		return r;
	}

	// f[^]Jn邽߂̕ϐs
	transfered_data_num[localmatchnum] = 0;

	r = Fcad416_StatusCheck();

	return 0;
}

static int Fcad416_StatusCheck( void )
{
	int r;
	unsigned char	ad_status[2];
	sleep( 1 );
	r = libusb_control_transfer(devh, CTRL_IN, Get_Status, 0, 0, ad_status, 2, 0);
	if (r < 0){
		printf("Status = %2x,ExtraStatus = %2x\n",ad_status[0],ad_status[1]);
		return r;
	}
	return ad_status[0];
}

static int Fcad416_Get_OneScan( int last_channel )
{
	int r, length, actual_length;
	unsigned char ad_data[32], ad_status[2];
	r = libusb_control_transfer(devh, CTRL_OUT, Get_One_Scan, (unsigned short)((last_channel << 8) | 0), 0, ad_status, 0, 0);
	if (r < 0){
		fprintf(stderr,"Get one scan has error\n");
		return r;
	}
//	printf("r = %d\n",r);				// pass above function with no error
	length = (last_channel + 1) * 2;
	r = libusb_bulk_transfer(devh, 0x86, ad_data, length, &actual_length, 0);
	printf("actual_length = %d , r = %d\n",actual_length , r);
	if (r < 0){
		fprintf(stderr,"Get onescan has error\n");
		return r;
	}
	for (r = 0 ; r <= last_channel ; r++){
		length = ad_data[r * 2] + (ad_data[r * 2 + 1] << 8);
		printf("%4x ",length);
	}
	printf("\n");
	return 0;
}

int Fcad416_Get_Status(int *post_sampled_data, int *pre_sampled_data, int get_status[])
{
	int	r, localmatchnum, pritriggerbufcount, loop;
	unsigned char	ctrl_buf[64], ad_status[2];

	localmatchnum = 0;									// As master mode only


		// RoCg̒lƂēԂĂiPʂ̓oCgj
	r = libusb_control_transfer(devh, CTRL_IN, Get_Pre_Count, 0, 0, ad_status, 3, 0);
	if (r < 0){
		fprintf(stderr,"Get status error occured\n");
		return r;
	}
	pritriggerbufcount = 0L;										// lOɁI
	for(loop = 0 ; loop < 3 ; loop++)
		pritriggerbufcount |= ctrl_buf[loop] << (loop * 8);
	pritriggerbufcount /= 2;												// [hPʂ̐lɕϊ
	pre_sampled_chnum[localmatchnum] = (pritriggerbufcount / channels[localmatchnum]);	// `lPʂ̃f[^ɂ
	*pre_sampled_data = pre_sampled_chnum[localmatchnum];				// [U[ɂʂ񍐂
		// QoCg̒lƂēԂĂiPʂ̓oCgj
	r = libusb_control_transfer(devh, CTRL_IN, Get_Status, 0, 0, ad_status, 2, 0);

	*post_sampled_data = transfered_data_num[localmatchnum] / channels[localmatchnum];	// `lPʂ̃f[^ɂ
	Image_Status[localmatchnum][0] = ctrl_buf[0];
	Image_Status[localmatchnum][1] = ctrl_buf[1];
	get_status[0] = Image_Status[localmatchnum][0];
	get_status[1] = Image_Status[localmatchnum][1];

	return NO_ERROR;											// ̃jbg̓Xbhx~Ԃ
}

static int Fcad416_Get_Status_In_Samp( unsigned char Aux_Out )
{
	int	loop, r, localmatchnum;
	unsigned char	ctrl_buf[64];

	localmatchnum = 0;												// As master mode only

	r = libusb_control_transfer(devh, CTRL_IN, Get_Status_InSamp, (unsigned char)Aux_Out, 0, ctrl_buf, 11, 0);

		Image_Status[localmatchnum][0] = ctrl_buf[0];						// {Xe[^X
		Image_Status[localmatchnum][1] = ctrl_buf[1];						// gXe[^X
		Image_Aux_in[localmatchnum] = ctrl_buf[2];						// ėp̓f[^
		ServiceRequestStatus[localmatchnum] = ctrl_buf[3];				// T[rXNGXg͂Rs[
		getdata_number = 0L;
		for (loop = 0 ; loop < 4 ; loop++)
			getdata_number |= (int)((ctrl_buf[loop + 4]) << (loop * 8));
		fifodata_number = 0L;
		for (loop = 0 ; loop < 3 ; loop++)
			fifodata_number |= (int)((ctrl_buf[loop + 8]) << (loop * 8));
		fifodata_number++;
		fifodata_number /= 2;						// ǂݏo\ȑf[^擾
	return r;
}

static int Fcad416_Get_Data( int sampnum )				// Parameter is datanum for channels
{
	int r, rr, length, actual_length, loop, localmatchnum;
	unsigned char	ctrl_buf[64];

	localmatchnum = 0;
	length = channels[localmatchnum] * sampnum * 2;

	for (loop = 0 ; loop < 4 ; loop++)
		ctrl_buf[loop] = ((length >> (loop * 8)) && 0xff);
	r = libusb_control_transfer(devh, CTRL_OUT, Read_Fifo_Data, 0, 0, ctrl_buf, 4, 0);
	if ( r < 0){
		fprintf(stderr,"Read Dll error\n");
		return r;
	}
	r = libusb_bulk_transfer(devh, 0x86, AD_ReadBuf, length, &actual_length, 0);

	if (r < 0){
		fprintf(stderr,"Get Data has error\n");
		return r;
	}
	for (rr = 0 ; rr < sampnum ; rr++){
		for (r = 0 ; r < channels[localmatchnum] ; r++){
			length = AD_ReadBuf[(rr * channels[localmatchnum] + r) * 2] + (AD_ReadBuf[(rr * channels[localmatchnum] + r) * 2 + 1] << 8);
			printf("%05d,",length);
		}
		printf("\n");
	}
//	printf("\n");
	return 0;
}

int Fcad416_Set_Trigger( int trgger_mode, int trgger_source, int trgger_type, int trigger_level1, int trigger_level2, int Force_Stop)
{
	int	r , localmatchnum;
	unsigned char	ctrl_buf[64];

	localmatchnum = 0;											// Master mode only	
	if (trg_mode == 0){											// |XggK[hw
		if(Startmode[localmatchnum] & PRE_START){				// XvgK[h
			Startmode[localmatchnum] &= ~PRE_START;
			Startmode[localmatchnum] &= ~CLOCK_ONLY;
		}
		else{													// X|XggK[h
			Startmode[localmatchnum] |= POST_START;
			Startmode[localmatchnum] &= ~CLOCK_ONLY;
		}
	}
	else if (trg_mode == 1){									// vgK[hw
		Startmode[localmatchnum] |= PRE_START;
		Startmode[localmatchnum] &= ~CLOCK_ONLY;
	}
	else if (trg_mode == 2)										// NbNI[w
		Startmode[localmatchnum] |= CLOCK_ONLY;
	else
		return ERR_TRIGMODE;

	if (RotarySwVal[localmatchnum] != 0){						// }X^[X[u[h
		if (trg_source == 0)
			TrigValue[localmatchnum] = SOFT_TRIG;				// gK[h\tggKȂX[u
		else if ((trg_source == 1) || (trg_source == 2))
			TrigValue[localmatchnum] = EXT_TRIG;				// ȊȌꍇ͊OgKƂ
		else
			return ERR_TRIGSOURCE;								// gK\[Xw肪ُ
	}
	else if (trg_source == 0)
		TrigValue[localmatchnum] = SOFT_TRIG;
	else if (trg_source == 1) // analog trig
		TrigValue[localmatchnum] = ANALOG_TRIG;
	else if (trg_source == 2) // external trig
		TrigValue[localmatchnum] = EXT_TRIG;
	else 
		return ERR_TRIGSOURCE;

	if (RotarySwVal[localmatchnum] != 0)						// X[u[h
																// X[ujbg疳ŗGbW
		TrigValue[localmatchnum] |= EDGE_TRIG;
	else if(trg_source == 0)
		TrigValue[localmatchnum] |= SOFT_TRIG;
	else if (trg_source == 1){	// analog trigger
		if (trg_type == 0)
			TrigValue[localmatchnum] |= EDGE_TRIG;
		else if (trg_type == 1) // analog trig
			TrigValue[localmatchnum] |= EDGE_TRIG + TRIG_PLUS;
		else if (trg_type == 2) // external trig
			;
		else if (trg_type == 3) // external trig
			TrigValue[localmatchnum] |= TRIG_PLUS;
		else if (trg_type == 4) // external trig
			TrigValue[localmatchnum] |= OUT_RANGE;
		else if (trg_type == 5) // external trig
			TrigValue[localmatchnum] |= IN_RANGE;
		else if (trg_type == 6) // external trig
			TrigValue[localmatchnum] |= DUAL_SLOPE_MINUS;
		else if (trg_type == 7) // external trig
			TrigValue[localmatchnum] |= DUAL_SLOPE_PLUS;
		else 
			return ERR_TRIGTYPE;
	}
	else if(trg_source == 2){	// external trigger
		if (trg_type == 0)
			TrigValue[localmatchnum] |= EDGE_TRIG;
		else if (trg_type == 1)
			TrigValue[localmatchnum] |= EDGE_TRIG + TRIG_PLUS;
		else 
			return ERR_TRIGTYPE;
	}
	else
		return ERR_TRIGSOURCE;

	if (trig_level1 >= trig_level2)
		return ERR_TRIG1andTRIG2;

	if (((trg_type == 4) | (trg_type == 5)) & (trig_level2 == (trig_level1 + 1)))
		return ERR_TRIG1andTRIG2;

	if (Force_Stop == ON)
		Startmode[localmatchnum] |= EXT_STOP_ENABLE;
	else
		Startmode[localmatchnum] &= ~EXT_STOP_ENABLE;
		// trigval(Sdat[2]) triglevel1(Sdat[3]) triglevel2(Sdat[4])
	r = libusb_control_transfer(devh, CTRL_OUT,Set_Trigger , (TrigValue[localmatchnum] + (trig_level1 << 8)), trig_level2, (unsigned char *)ctrl_buf, 0, 0);
	if (r != 0)
		return ERR_NO416;

	CheckParameterSet[localmatchnum] |= Set_TriggerMode;
	TrigSet[localmatchnum] = Trigged;
	if(Scan_Set[localmatchnum] == Set_FirstCh){		// XL̓ZbgĂăgKZbgꍇ
													// ̎_ōēxXLZbgiSetOrderjR}h𔭍s
		Scan_Set[localmatchnum] = Set_FirstCh_Already;
	r = libusb_control_transfer(devh, CTRL_OUT,Set_Order , (SCAN_0 + (unsigned short)((channels[localmatchnum] - 1) << 8)), 0, (unsigned char *)ctrl_buf, 0, 0);
		if (r != 0)
			return ERR_NO416;
	}
	return NO_ERROR;
}

int Fcad416_Set_SampCh(int scan_channel, int scan_ch_order[], int inputrange)
{
	unsigned char i;
	int	r , localmatchnum;
	unsigned char	ctrl_buf[64];

	localmatchnum = 0;							// Master mode only
	if ((scan_channel <= 0) || (MAX_CH < scan_channel))
		return ERR_AD_CHANNEL_OVER;
	
	// chnum(Sdat[2]) scanorder(Sdat[3]) range(Sdat[4])(=LSB,MSB,LSB)
	for (i = 0; i < scan_channel; i++){
	r = libusb_control_transfer(devh, CTRL_OUT,Set_Channel , (i + (scan_ch_order[i] << 8)), inputrange, (unsigned char *)ctrl_buf, 0, 0);
	}
	CheckParameterSet[localmatchnum] |= Set_InputRange;

	if(TrigSet[localmatchnum] == Not_Trigged){
		Scan_Set[localmatchnum] = Set_FirstCh;							// gKZbgsɁA̕ϐmF
		return ERR_CHSET_WithoutTRIG;									// ZbgI[_[R}h𔭍s
	}

	channels[localmatchnum] = scan_channel;

	// startscan(Sdat[2]) endscan(Sdat[3])

	r = libusb_control_transfer(devh, CTRL_OUT,Set_Order , (SCAN_0 + ((channels[localmatchnum] - 1) << 8)), 0, (unsigned char *)ctrl_buf, 0, 0);
	if (r < 0)
		return r;
	Scan_Set[localmatchnum] = Set_FirstCh_Already;
	CheckParameterSet[localmatchnum] |= Set_FirstChannel | Set_LastChannel;

	return NO_ERROR;
}

int Fcad416_Set_Clock(int clock_source, int clk_set_mode, int clk_time_unit, int clock_period, int exclock_freq)
{
	int	r , loop, localmatchnum;
	double	freq, ScantimeValue;
	unsigned char	ctrl_buf[64];

	localmatchnum = 0;
	// ȉ̓}X^[jbg̏ꍇ̃V[PXŁA͂PUDRWSlggp̂݊YrbgZbgĂ
	// ]āAXL^C~OZbg̒ł͂̃rbg̗Lɂď킯΂悢ɂȂB
	if (clock_source == Int20M)
		ClockValue[0] = INT_CLOCK20;
	else if (clock_source == Int16M)
		ClockValue[0] = INT_CLOCK16;
	else if ((clock_source == ExtMinus20) | (clk_source == ExtMinus16))
		ClockValue[0] = EXT_CLOCK_MINUS;
	else if ((clock_source == ExtPlus20) | (clk_source == ExtPlus16))
		ClockValue[0] = EXT_CLOCK_PLUS;
	else 
		return ERR_CLOCKSOURCE;

	if (clk_set_mode == Set_Period){											// w胂[h
		// }X^[jbgւ̐ݒlvZs
		// ܂AXL̐iXL^C~O~`lȏ̒lǂj؂
		if (clock_period == Clk_S)						// Pʂ͕bȂ̂łPOO{
			ScantimeValue = ceil(clk_time_unit * 1000000 / channels[0]);
		else if (clock_period == Clk_mS)					// Pʂ͂bȂ̂Ő{
			ScantimeValue = ceil(clk_time_unit * 1000 / channels[0]);
		else if (clock_period == Clk_uS)					// Pʂ͂bȂ̂ł̂܂
			ScantimeValue = ceil(clk_time_unit / channels[0]);
		else
			return ERR_CLOCKPERIOD;						// G[񍐂ďIB
		// `l̃TvO^CiPʂ̓ʂrj߂
		if (ScantimeValue < Max_ScanSpeed)
			return ERR_SCAN_CLOCK_VALUE;				// G[񍐂ďIB
		// ̌ʂQlɂĎۂ̃TvOK肷邽߂̂P`l̃NbNvZB
		if (clock_period == Clk_S){						// ԒPʂb
			if ((int)(ScantimeValue * channels[0]) > (clk_time_unit * 1000000))
				freq = 1.0 / ScantimeValue;
			else
				freq = 1.0 / (clk_time_unit * 1000000 / channels[0]);
		}
		else if (clock_period == Clk_mS){					// ԒPʂb
			if ((int)(ScantimeValue * channels[0]) > (clk_time_unit * 1000))
				freq = 1.0 / ScantimeValue;
			else
				freq = 1.0 / (clk_time_unit * 1000 / channels[0]);
		}
		else{											// ԒPʂʕb
			if ((int)(ScantimeValue * channels[0]) > clk_time_unit)
				freq = 1.0 / ScantimeValue;
			else
				freq = 1.0 / (clk_time_unit / channels[0]);
		}
		// ߂XL^CƂTvO番ČvZ
			freq *= 1000000;								// ܂gvZ

		if (clk_source == Int20M)							// NbNQOlg̏ꍇ
			PacerClock[0] = (int)ceil(20000000 / freq) * channels[0];
		else if(clk_source == Int16M)						// NbNPUDRWSlg̏ꍇ
			PacerClock[0] = (int)ceil(16384000 / freq) * channels[0];
		// ONbN̏ꍇi/オj
		else{
			if ((int)(exclock_freq * channels[0]) > (int)freq)		// Pȏ̕䂪ݒłꍇ
				PacerClock[0] = (int)ceil(exclock_freq * channels[0] / freq);
			else if (exclock_freq > 0)									// 䂪P菬ȂĂ܂ꍇ
				PacerClock[0] = 1;							// IɂPɐݒ肷
			else
				return ERR_CLOCKPERIOD;
		}
	}
	else if (clk_set_mode == Set_Devide){									// w胂[h
		if (ClockValue[0] == INT_CLOCK20){				// NbNQOlg쎞
			if (clk_time_unit * channels[0] / 20.000000 < 4.0)
				return ERR_DEVIDE_VALUE;				// ŒłSʂr̃XL^Cȏ゠邱Ƃ
			else
				PacerClock[0] = clk_time_unit;
		}
		else if (ClockValue[0] == INT_CLOCK16){		// NbNPUDRWSlg쎞
			if (clk_time_unit * channels[0] / 16.384000 < 4.0)
				return ERR_DEVIDE_VALUE;
			else
				PacerClock[0] = clk_time_unit;
		}
		else
			PacerClock[0] = clk_time_unit;				// X[ujbg̏ꍇl
	}
	else
		return ERR_CLOCKSETMODE;

	for (loop = 0 ; loop < 4 ; loop++)
		ctrl_buf[loop] = (unsigned char)(PacerClock[0] >> (loop * 8));

	r = libusb_control_transfer(devh, CTRL_OUT,Set_Clock , (ClockValue[0]), 0, ctrl_buf, 4, 0);
	if ( r < 0 ){
		fprintf(stderr,"Clock set fail\n");
		return r;
	}
	CheckParameterSet[localmatchnum] |= Set_SelectClock | Set_ClockDiv;
	return NO_ERROR;
}

int	Fcad416_Set_ScanSpeed(int ScanclkMode , int Scan_Speed)
{
	// X[ujbg͊ONbNgp̂łPUlgNbNgpĂĂÅ֐
	// fłȂBƌƁA܂ł̃R[hł̓X[ujbg͖ɓNbNQOlg
	// 삵ĂɂȂB
	int	r , localmatchnum;
	unsigned char	ctrl_buf[64];

	localmatchnum = 0;											// Fix mastermode only
	if ((ScanclkMode < 0) || (ScanclkMode > 1))
		return ERR_CLOCKSETMODE;								// w@ԈĂ̂ŃG[
	// TvONbNƂēPUDRWSlggp̂AȊÕNbNgp̂
	// Fcad416_Set_Clock֐̒ClockValue[localmatchnum]ϐɃZbgĂB
	if (ScanclkMode == 0){									// wȂ̂ŕvZ
		if ( Scan_Speed < Max_ScanSpeed)										// 4uSőTvO
			return ERR_SCAN_CLOCK_VALUE;
		else if (( Scan_Speed > (65536/20)) && ((ClockValue[localmatchnum] & INT_CLOCK16) != INT_CLOCK16))
			return ERR_SCAN_CLOCK_VALUE;						// ݒl𒴂̂ŃG[
		else if (( Scan_Speed > (65536/16)) && ((ClockValue[localmatchnum] & INT_CLOCK16) == INT_CLOCK16))
			return ERR_SCAN_CLOCK_VALUE;						// G[

		if (ClockValue[localmatchnum] & INT_CLOCK16)
			ScanClockValue[localmatchnum] = (unsigned short)ceil((Scan_Speed * 16.384));	// PUDRWSlggp
		else
			ScanClockValue[localmatchnum] = (unsigned short)(Scan_Speed * 20);	// QOlggp
	}
	else{														// wȂ̂ŃG[`FbNĂ
		if (ClockValue[localmatchnum] & INT_CLOCK16){			// NbNPUDRWSlg
			if ((Scan_Speed < 66) || (Scan_Speed > 65535))
				return ERR_DEVIDE_VALUE;
			ScanClockValue[localmatchnum] = (unsigned short)Scan_Speed;
		}
		else{													// NbNQOlg
			if ((Scan_Speed < 80) || (Scan_Speed > 65535))
				return ERR_DEVIDE_VALUE;
			ScanClockValue[localmatchnum] = (unsigned short)Scan_Speed;
		}
	}
	
		// scanvalue(Sdat[2::3])
	r = libusb_control_transfer(devh, CTRL_OUT,Set_ScanSpeed , (ScanClockValue[localmatchnum]), 0, (unsigned char *)ctrl_buf, 0, 0);
	if (r < 0)
		return r;
		
	CheckParameterSet[localmatchnum] |= Set_SampClk;
	return NO_ERROR;
}

int Fcad416_Set_InpMode( int input_mode, int analog_code)
{
	int	r , localmatchnum;
	unsigned char	ctrl_buf[64];

	localmatchnum = 0;								// Master mode only
	if (input_mode == 0)
		InpMode[localmatchnum] &= ~DIFF_INPUT;		// VOGhꍇ̓rbgNA
	else if (input_mode == 1)
		InpMode[localmatchnum] |= DIFF_INPUT;
	else
		return ERR_INPMODE;

	if (analog_code == 0)
		InpMode[localmatchnum] &= ~COMP;			// oCi[hQ̕␔[hrbgNA
	else if (analog_code == 1)
		InpMode[localmatchnum] |= COMP;
	else 
		return ERR_ADCODE;
	
		// mode(Sdat[2])
	r = libusb_control_transfer(devh, CTRL_OUT,Set_Inpmode , (InpMode[localmatchnum]), 0, (unsigned char *)ctrl_buf, 0, 0);
	if (r < 0)
		return r;

	CheckParameterSet[localmatchnum]  |= Set_AdCode;
	return NO_ERROR;
	
}

int Fcad416_Set_TransferSize( int t_size)
{
	int	localmatchnum;

	localmatchnum = 0;
	if ((t_size < 1) | (MAX_TRANSFER_NO < t_size))
		return ERR_TRANSFERSIZE;

	if(t_size == 1)
		transfer_size[localmatchnum] = TRANSFER_SIZE_BASE * 2;
	else if(t_size == 2)
		transfer_size[localmatchnum] = TRANSFER_SIZE_BASE * 4;
	else if(t_size == 3)
		transfer_size[localmatchnum] = TRANSFER_SIZE_BASE * 8;
	else if(t_size == 4)
		transfer_size[localmatchnum] = TRANSFER_SIZE_BASE * 16;
	else if(t_size == 5)
		transfer_size[localmatchnum] = TRANSFER_SIZE_BASE * 32;
	else if(t_size == 6)
		transfer_size[localmatchnum] = TRANSFER_SIZE_BASE * 64;
	else if(t_size == 7)
		transfer_size[localmatchnum] = TRANSFER_SIZE_BASE * 128;
	else if(t_size == 8)
		transfer_size[localmatchnum] = TRANSFER_SIZE_BASE * 256;
	else if(t_size == 9)
		transfer_size[localmatchnum] = TRANSFER_SIZE_BASE * 512;
	else if(t_size == 10)
		transfer_size[localmatchnum] = TRANSFER_SIZE_BASE * 1024;
	else if(t_size == 11)
		transfer_size[localmatchnum] = TRANSFER_SIZE_BASE * 2048;
	else if(t_size == 12)
		transfer_size[localmatchnum] = TRANSFER_SIZE_BASE * 4096;
	else if(t_size == 13)
		transfer_size[localmatchnum] = TRANSFER_SIZE_BASE * 8192;
	else if(t_size == 14)
		transfer_size[localmatchnum] = TRANSFER_SIZE_BASE * 16384;
	else if(t_size == 15)
		transfer_size[localmatchnum] = TRANSFER_SIZE_BASE * 32768;
	else if(t_size == 16)
		transfer_size[localmatchnum] = TRANSFER_SIZE_BASE * 65536;
	else if(t_size == 17)
		transfer_size[localmatchnum] = TRANSFER_SIZE_BASE * 131072;
	else if(t_size == 18)
		transfer_size[localmatchnum] = TRANSFER_SIZE_BASE * 262144;
	else if(t_size == 19)
		transfer_size[localmatchnum] = TRANSFER_SIZE_BASE * 524288;
	else if(t_size == 20)
		transfer_size[localmatchnum] = TRANSFER_SIZE_BASE * 1048576;
	else if(t_size == 21)
		transfer_size[localmatchnum] = TRANSFER_SIZE_BASE * 2097152;
	else if(t_size == 22)
		transfer_size[localmatchnum] = TRANSFER_SIZE_BASE * 4194304;
	else
		transfer_size[localmatchnum] = TRANSFER_SIZE_BASE * 8388608;
	return NO_ERROR;
}

int Fcad416_Set_Memory( int mem)
{
	int	r , localmatchnum;
	unsigned char	ctrl_buf[64];

	localmatchnum = 0;					// Master mode only
	if ((mem < 0) || (MAX_MEM_NO < mem))
		return ERR_MEMSIZE;
	
	mem &= 0x07;						// svrbg̃}XN
	mem |= HalfFullSize[localmatchnum];	// n[ttTCYݒ
	RingBufferSize[localmatchnum] = mem;	// Oobt@TCYZbg


		// code(Sdat[2])
	r = libusb_control_transfer(devh, CTRL_OUT,Set_Memory , mem, 0, (unsigned char *)ctrl_buf, 0, 0);
	if (r < 0)
		return r;
	return NO_ERROR;
}

static int Fcad416_GetParams( void )
{
	int r;
	unsigned char ad_data[64];
	char	substr[10],strings[128];
	r = libusb_control_transfer(devh, CTRL_IN, Inquiry_Command , 0, 0, (char *)ad_data, 62, 0);
	if (r < 0){
		fprintf(stderr,"Get params has error\n");
		return r;
	}
	for(r = 0 ; r < 128 ; r++)
		strings[r] = 0;
	for(r = 0 ; r < 62 ; r++){
		sprintf(substr,"%02x ",ad_data[r]);
		strcat(strings,substr);
	}
	strcat(strings,"\n");
	printf( strings );
	return 0;
}

void ParameterSet_Fcad416( void )
{
	// p[^ݒ{^
	int r, loop, Stop_Func;

	Stop_Func = 0;

	trg_mode = 0;				// |XggKAvgKʁiL^j
	trg_source = IMTRIG;			// gKށi\tgAAiOAfW^j
	trg_type = 0;					// gKށiGbWAxAfAGbWj
	trig_level1 = 127;
	trig_level2 = 128;
	if((r = Fcad416_Set_Trigger( trg_mode, trg_source, trg_type, trig_level1, trig_level2, Stop_Func)) != 0){
		fprintf(stderr,"Trigger Fail\n");
		return;
	}

	board_no[0] = 0;
	no_ch = 16;
	Memory_size[0] = 0;							// post trigger mode

	for(loop = 0 ; loop < no_ch ; loop++)
		scan_order[loop] = loop;
	if((r = Fcad416_Set_SampCh(no_ch, scan_order, range[0])) != 0){
		fprintf(stderr,"Set sampch fail\n");
		return;
	}

	clk_source = 0;		// 20MHzsource
	set_mode = 1;		// value by devide
	time_unit = 1280;	// defive value for 64uS
	clk_period = 0;		// not use in devide mode
	exclk_freq = 0;		// not use internal clock mode
	if((r = Fcad416_Set_Clock( clk_source, set_mode, time_unit, clk_period, exclk_freq)) != 0){
		fprintf(stderr,"Set clock fail\n");
		return;
	}

												// ̏ԁiSet_Clock->Set_ScanSpeedjȂƃXLNbN
												// vZɍsȂiXLNbŇvZɃTvONbN
												// lgpĂ邽߁j
	ScanClockMode = 1;							// set by devide value
	Scanclockvalue = 80;						// deviden

	if((r = Fcad416_Set_ScanSpeed( ScanClockMode , Scanclockvalue)) != 0){
		fprintf(stderr,"ScanSpeed set fail\n");
		return;
	}

	inp_mode = 0;						// single mode input
	ad_code = 0;						// code is binary

	if((r = Fcad416_Set_InpMode( inp_mode, ad_code)) != 0){
		fprintf(stderr,"SetInpMode fail\n");
		return;
	}

	transfersize = 23;						// ]obt@TCY 8MWord
	if((r = Fcad416_Set_TransferSize( transfersize )) != 0){
		fprintf(stderr,"Transfersize set fail.\n");
		return;
	}

	if((r = Fcad416_Set_Memory( Memory_size[board_no[0]])) != 0){
		fprintf(stderr,"PritriggermemorySize set fail\n");
		return;
	}

	return;
}

/****************************************************************************/
/*																			*/
/*							Sample program main								*/
/*																			*/
/****************************************************************************/
int main(void)
{
	int	rest_data, once_data;

	int r = 1;

	r = libusb_init(NULL);
	if (r < 0) {
		fprintf(stderr, "failed to initialise libusb\n");
		exit(1);
	}
	r = find_flexcore_device();
	if (r < 0) {
		fprintf(stderr, "Could not find/open device\n");
		exit(1);
	}
	printf("device founded\n");
	r = libusb_claim_interface(devh, 0);		// Use interface number 0 only with FCAD416
	if (r < 0) {
		fprintf(stderr, "usb_claim_interface error %d\n", r);
		exit(1);
	}
	printf("Claimed interface\n");

	r = Hard_Init();				// Initialize Fcad416 and test unit ID
	if (r < 0){
		fprintf(stderr,"Fcad416 cannot use.\n");
		goto out;
	}

	ParameterSet_Fcad416();			// Parameter set for test pattern


	post_samp = 4096;
	r = Fcad416_Start_Samp( post_samp );
	if (r < 0){
		fprintf(stderr,"Sampling start fail.\n");
		goto out;
	}

	do{
		r = Fcad416_StatusCheck();				// loop until sampling end detected
		if (r < 0){
			fprintf(stderr,"Sampling end fail.\n");
			goto out;
		}
	}
	while((r & 0x80) != 0x80);					// Till status register0's bit7 set(Sampling end)
	rest_data = post_samp * channels[0];
	do{
	if (rest_data > BlockSize)
		once_data = BlockSize;
	else
		once_data = rest_data;
	r = Fcad416_Get_Data( once_data / channels[0]);
	rest_data -= once_data;
	}while(rest_data != 0);
/*	Fcad416_Get_OneScan( 13 );					// Last channel EQU 13(ch0,ch1,ch2, .. ,ch13)
	r = Fcad416_StatusCheck();*/				// loop until sampling end detected

	printf("shutting down...\n");
	
	libusb_release_interface(devh, 0);
out:
	fprintf(stderr,"Device has closed \n");
	libusb_close(devh);
	libusb_exit(NULL);
	return r >= 0 ? r : -r;
}

